package com.mycompany.app;

import com.couchbase.client.ClusterManager;
import com.couchbase.client.CouchbaseClient;
import com.couchbase.client.CouchbaseConnectionFactory;
import com.couchbase.client.CouchbaseConnectionFactoryBuilder;
import com.google.common.collect.Lists;
import net.spy.memcached.FailureMode;
import net.spy.memcached.internal.OperationFuture;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URI;
import java.util.List;
import java.util.concurrent.ExecutionException;

public
class CouchbaseUtil {
    private static final List<String> URI_ADDRESS = Lists.newArrayList(
            "http://147.54.160.217:8091/pools",
            "http://147.54.160.218:8091/pools"
    );

    public final Logger LOGGER
            = LoggerFactory.getLogger(CouchbaseUtil.class.getSimpleName());

    private static final String USER_NAME = "Administrator";

    private static final String BUCKET_NAME = "default";

    public static final CouchbaseUtil INSTANCE = new CouchbaseUtil();

    private CouchbaseClient client;

    private ClusterManager manager;

    private
    CouchbaseUtil() {
        try {
            List<URI> hosts = Lists.newArrayList();
            for (String uriAddres : URI_ADDRESS) {
                hosts.add(URI.create(uriAddres));
            }

            CouchbaseConnectionFactoryBuilder cfb
                    = new CouchbaseConnectionFactoryBuilder();
            CouchbaseConnectionFactory cf = cfb.buildCouchbaseConnection(hosts,
                                                                         BUCKET_NAME,
                                                                         USER_NAME,
                                                                         "");
            final FailureMode failureMode = cf.getFailureMode();
            LOGGER.warn("failureMode: {}",
                        failureMode.toString());
            manager = cf.getClusterManager();
            client = new CouchbaseClient(cf);
        } catch (IOException e) {
            e.printStackTrace();
            throw new IllegalStateException(e.getMessage());
        }
    }

    public
    void write(
            String key,
            Object value) {
        final OperationFuture<Boolean> set = client.set(key,
                                                        value);
        try {
            set.get();
        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
            throw new IllegalStateException(e.getMessage());
        } catch (RuntimeException e) {
            write(key,
                  value);
        }
    }
    public
    void flushBucket() {
        try {
            manager.flushBucket(BUCKET_NAME);
        } catch (RuntimeException e) {
            e.printStackTrace();
            throw new IllegalStateException(e.getMessage());
        }
    }
}